@@ -13,6 +13,8 @@ import android.view.View
 import android.view.ViewGroup
 import android.view.accessibility.AccessibilityEvent
 import android.view.accessibility.AccessibilityNodeInfo
+import android.view.accessibility.AccessibilityNodeInfo.CHECKED_STATE_FALSE
+import android.view.accessibility.AccessibilityNodeInfo.CHECKED_STATE_TRUE
 import android.view.accessibility.AccessibilityNodeProvider
 import android.view.accessibility.AccessibilityRecord
 import android.widget.EditText
@@ -128,6 +130,7 @@ class AccessibilityTest : BaseSessionTest() {
         mainSession.accessibility.view = view
 
         // Set up an external delegate that will intercept accessibility events.
+        @Suppress("DEPRECATION") // TYPE_ANNOUNCEMENT is deprecated but triggered by live regions
         sessionRule.addExternalDelegateUntilTestEnd(
             EventDelegate::class,
             { newDelegate ->
@@ -363,7 +366,7 @@ class AccessibilityTest : BaseSessionTest() {
         loadTestPage("test-text-entry-node")
         waitForInitialFocus()
 
-        mainSession.evaluateJS("document.querySelector('input[aria-label=Name]').focus()")
+        mainSession.evaluateJS("document.querySelector('input[aria-label=Naame]').focus()")
 
         sessionRule.waitUntilCalled(object : EventDelegate {
             @AssertCalled(count = 1)
@@ -378,7 +381,7 @@ class AccessibilityTest : BaseSessionTest() {
                 assertThat(
                     "Hint has field name",
                     node.extras.getString("AccessibilityNodeInfo.hint"),
-                    equalTo("Name description"),
+                    equalTo("Naame description"),
                 )
             }
         })
@@ -521,7 +524,14 @@ class AccessibilityTest : BaseSessionTest() {
                 var nodeId = getSourceId(event)
                 var node = createNodeInfo(nodeId)
                 assertThat("Event's checked state matches", event.isChecked, equalTo(checked))
-                assertThat("Checkbox node has correct checked state", node.isChecked, equalTo(checked))
+
+                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.BAKLAVA) {
+                    val checkedState = if (checked) CHECKED_STATE_TRUE else CHECKED_STATE_FALSE
+                    assertThat("Checkbox node has correct checked state", node.checked, equalTo(checkedState))
+                } else {
+                    @Suppress("DEPRECATION")
+                    assertThat("Checkbox node has correct checked state", node.isChecked, equalTo(checked))
+                }
             }
         })
     }
@@ -980,7 +990,14 @@ class AccessibilityTest : BaseSessionTest() {
                 assertThat("Checkbox node is checkable", node.isCheckable, equalTo(true))
                 assertThat("Checkbox node is clickable", node.isClickable, equalTo(true))
                 assertThat("Checkbox node is focusable", node.isFocusable, equalTo(true))
-                assertThat("Checkbox node is not checked", node.isChecked, equalTo(false))
+
+                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.BAKLAVA) {
+                    assertThat("Checkbox node is not checked", node.checked, equalTo(CHECKED_STATE_FALSE))
+                } else {
+                    @Suppress("DEPRECATION")
+                    assertThat("Checkbox node is not checked", node.isChecked, equalTo(false))
+                }
+
                 assertThat("Checkbox node has correct role", node.text.toString(), equalTo("many option"))
                 assertThat(
                     "Hint has description",
@@ -1109,6 +1126,12 @@ class AccessibilityTest : BaseSessionTest() {
         loadTestPage("test-live-region")
         waitForInitialFocus()
 
+        val rootNode = createNodeInfo(View.NO_ID)
+        assertThat("Document has 1 child", rootNode.childCount, equalTo(1))
+
+        val liveRegion = createNodeInfo(rootNode.getChildId(0))
+        assertThat("First node is a label", liveRegion.viewIdResourceName.toString(), equalTo("to_change"))
+
         mainSession.evaluateJS("document.querySelector('#to_change').textContent = 'Hello';")
         sessionRule.waitUntilCalled(object : EventDelegate {
             @AssertCalled(count = 1)
@@ -1202,6 +1225,19 @@ class AccessibilityTest : BaseSessionTest() {
         })
     }
 
+    @Test fun testLiveRegionStatus() {
+        loadTestPage("test-live-region-status")
+        waitForInitialFocus()
+
+        mainSession.evaluateJS("document.querySelector('#status').textContent = 'hello';")
+        sessionRule.waitUntilCalled(object : EventDelegate {
+            @AssertCalled(count = 1)
+            override fun onAnnouncement(event: AccessibilityEvent) {
+                assertThat("Announcement is correct", event.text[0].toString(), equalTo("hello"))
+            }
+        })
+    }
+
     private fun screenContainsNode(nodeId: Int): Boolean {
         var node = createNodeInfo(nodeId)
         var nodeBounds = Rect()
@@ -1392,13 +1428,6 @@ class AccessibilityTest : BaseSessionTest() {
 
     @Test
     fun autoFill_navigation() {
-        // Fails with BFCache in the parent.
-        // https://bugzilla.mozilla.org/show_bug.cgi?id=1715480
-        sessionRule.setPrefsUntilTestEnd(
-            mapOf(
-                "fission.bfcacheInParent" to false,
-            ),
-        )
         fun countAutoFillNodes(
             cond: (AccessibilityNodeInfo) -> Boolean =
                 { it.className == "android.widget.EditText" },
@@ -1465,7 +1494,7 @@ class AccessibilityTest : BaseSessionTest() {
         )
 
         // Now wait for the nodes to reappear.
-        mainSession.goBack()
+        mainSession.goBack(false)
         waitForInitialFocus()
         waitForAutoFillNodes()
         assertThat(
@@ -1661,6 +1690,8 @@ class AccessibilityTest : BaseSessionTest() {
     @Test
     fun testRemoteIframeTree() {
         testIframeTree(REMOTE_IFRAME)
+        // disabled for frequent failures - on Bug 1769324
+        assumeThat(sessionRule.env.isX86, equalTo(false))
     }
 
     @Test
@@ -2162,13 +2193,17 @@ class AccessibilityTest : BaseSessionTest() {
         loadTestPage("test-text-entry-node")
         waitForInitialFocus()
 
-        mainSession.evaluateJS("document.querySelector('input[aria-label=Name]').focus()")
+        mainSession.evaluateJS("document.querySelector('input[aria-label=Naame]').focus()")
         sessionRule.waitUntilCalled(object : EventDelegate {
             @AssertCalled(count = 1)
             override fun onFocused(event: AccessibilityEvent) {}
+
+            // Focus fires a caret moved event which produces this event.
+            @AssertCalled(count = 1)
+            override fun onTextSelectionChanged(event: AccessibilityEvent) {}
         })
 
-        mainSession.evaluateJS("document.querySelector('input[aria-label=Name]').value = 'Tobiasas'")
+        mainSession.evaluateJS("document.querySelector('input[aria-label=Naame]').value = 'Tobiasas'")
 
         sessionRule.waitUntilCalled(object : EventDelegate {
             @AssertCalled(count = 1)
