@@ -9,6 +9,7 @@ package org.mozilla.geckoview;
 import android.annotation.SuppressLint;
 import android.app.ActivityManager;
 import android.app.PendingIntent;
+import android.app.Service;
 import android.content.ComponentName;
 import android.content.Context;
 import android.content.Intent;
@@ -40,6 +41,7 @@ import java.lang.annotation.Retention;
 import java.lang.annotation.RetentionPolicy;
 import java.util.List;
 import java.util.Map;
+import org.mozilla.gecko.Clipboard;
 import org.mozilla.gecko.EventDispatcher;
 import org.mozilla.gecko.GeckoAppShell;
 import org.mozilla.gecko.GeckoNetworkManager;
@@ -49,6 +51,7 @@ import org.mozilla.gecko.GeckoScreenOrientation.ScreenOrientation;
 import org.mozilla.gecko.GeckoSystemStateListener;
 import org.mozilla.gecko.GeckoThread;
 import org.mozilla.gecko.annotation.WrapForJNI;
+import org.mozilla.gecko.crashhelper.CrashHelper;
 import org.mozilla.gecko.process.MemoryController;
 import org.mozilla.gecko.util.BundleEventListener;
 import org.mozilla.gecko.util.DebugConfig;
@@ -183,6 +186,14 @@ public final class GeckoRuntime implements Parcelable {
       // Set settings that may have changed between last app opening
       GeckoAppShell.setIs24HourFormat(
           DateFormat.is24HourFormat(GeckoAppShell.getApplicationContext()));
+
+      // OnPrimaryClipChangedListener() won’t be triggered for a background
+      // application, so update the clipboard sequence number once the
+      // application returns to the foreground.
+      ThreadUtils.sGeckoHandler.post(
+          () -> {
+            Clipboard.updateSequenceNumber(GeckoAppShell.getApplicationContext());
+          });
     }
 
     @OnLifecycleEvent(Lifecycle.Event.ON_PAUSE)
@@ -194,6 +205,7 @@ public final class GeckoRuntime implements Parcelable {
       // Stop monitoring network status while inactive.
       GeckoNetworkManager.getInstance().stop();
       GeckoThread.onPause();
+      Clipboard.onPause();
     }
   }
 
@@ -266,33 +278,15 @@ public final class GeckoRuntime implements Parcelable {
    * @return SessionID to use for the request.
    */
   @SuppressLint("WrongThread") // for .isOpen() which is called on the UI thread
-  @WrapForJNI(calledFrom = "gecko")
-  private static @NonNull GeckoResult<String> serviceWorkerOpenWindow(final @NonNull String url) {
+  @UiThread
+  private static @NonNull GeckoResult<GeckoSession> serviceWorkerOpenWindow(
+      final @NonNull String url) {
     if (sRuntime != null && sRuntime.mServiceWorkerDelegate != null) {
-      final GeckoResult<String> result = new GeckoResult<>();
-      // perform the onOpenWindow call in the UI thread
-      ThreadUtils.runOnUiThread(
-          () -> {
-            sRuntime
-                .mServiceWorkerDelegate
-                .onOpenWindow(url)
-                .accept(
-                    session -> {
-                      if (session != null) {
-                        if (!session.isOpen()) {
-                          session.open(sRuntime);
-                        }
-                        result.complete(session.getId());
-                      } else {
-                        result.complete(null);
-                      }
-                    });
-          });
-      return result;
-    } else {
-      return GeckoResult.fromException(
-          new java.lang.RuntimeException("No available Service Worker delegate."));
+      ThreadUtils.assertOnUiThread();
+      return sRuntime.mServiceWorkerDelegate.onOpenWindow(url);
     }
+    return GeckoResult.fromException(
+        new java.lang.RuntimeException("No available Service Worker delegate."));
   }
 
   /**
@@ -327,9 +321,16 @@ public final class GeckoRuntime implements Parcelable {
             final String url = message.getString("url", "about:blank");
             serviceWorkerOpenWindow(url)
                 .then(
-                    (GeckoResult.OnValueListener<String, Void>)
-                        value -> {
-                          callback.sendSuccess(value);
+                    (GeckoResult.OnValueListener<GeckoSession, Void>)
+                        session -> {
+                          if (session == null) {
+                            callback.sendSuccess(null);
+                            return null;
+                          }
+                          if (!session.isOpen()) {
+                            session.open(sRuntime);
+                          }
+                          callback.sendSuccess(session.getId());
                           return null;
                         })
                 .exceptionally(
@@ -351,6 +352,32 @@ public final class GeckoRuntime implements Parcelable {
             } else {
               context.startService(i);
             }
+          } else if ("GeckoView:ServiceWorkerOpenWindow".equals(event)) {
+            final String url = message.getString("url", "about:blank");
+            serviceWorkerOpenWindow(url)
+                .then(
+                    (GeckoResult.OnValueListener<GeckoSession, Void>)
+                        session -> {
+                          if (session == null) {
+                            callback.sendSuccess(null);
+                            return null;
+                          }
+                          final boolean isOpen = session.isOpen();
+                          if (!isOpen) {
+                            session.open(sRuntime);
+                          }
+                          final GeckoBundle bundle = new GeckoBundle();
+                          bundle.putBoolean("isOpen", isOpen);
+                          bundle.putString("sessionId", session.getId());
+                          callback.sendSuccess(bundle);
+                          return null;
+                        })
+                .exceptionally(
+                    (GeckoResult.OnExceptionListener<Void>)
+                        error -> {
+                          callback.sendError(error + " Could not open tab.");
+                          return null;
+                        });
           }
         }
       };
@@ -371,6 +398,33 @@ public final class GeckoRuntime implements Parcelable {
     return null;
   }
 
+  private int[] startCrashHelper() {
+    final Context context = GeckoAppShell.getApplicationContext();
+    final CrashHelper.Pipes pipes = CrashHelper.createCrashHelperPipes(context);
+
+    if (pipes == null) {
+      Log.e(LOGTAG, "Could not create the crash reporter IPC pipes");
+      return new int[] {-1, -1};
+    }
+
+    try {
+      @SuppressWarnings("unchecked")
+      final Class<? extends Service> cls =
+          (Class<? extends Service>) Class.forName("org.mozilla.gecko.crashhelper.CrashHelper");
+      final Intent i = new Intent(context, cls);
+      final File minidumps = new File(context.getFilesDir(), "minidumps");
+      context.bindService(
+          i,
+          CrashHelper.createConnection(
+              pipes.mBreakpadServer, minidumps.getPath(), pipes.mListener, pipes.mServer),
+          Context.BIND_AUTO_CREATE);
+    } catch (final ClassNotFoundException e) {
+      Log.w(LOGTAG, "Couldn't find the crash helper class");
+    }
+
+    return new int[] {pipes.mBreakpadClient.detachFd(), pipes.mClient.detachFd()};
+  }
+
   /* package */ boolean init(
       final @NonNull Context context, final @NonNull GeckoRuntimeSettings settings) {
     if (DEBUG) {
@@ -382,6 +436,10 @@ public final class GeckoRuntime implements Parcelable {
       flags |= GeckoThread.FLAG_DEBUGGING;
     }
 
+    if (!settings.getLowMemoryDetection()) {
+      flags |= GeckoThread.FLAG_DISABLE_LOW_MEMORY_DETECTION;
+    }
+
     final Class<?> crashHandler = settings.getCrashHandler();
     if (crashHandler != null) {
       try {
@@ -437,6 +495,8 @@ public final class GeckoRuntime implements Parcelable {
       }
     }
 
+    final int[] fds = startCrashHelper();
+
     final GeckoThread.InitInfo info =
         GeckoThread.InitInfo.builder()
             .args(args)
@@ -444,6 +504,7 @@ public final class GeckoRuntime implements Parcelable {
             .flags(flags)
             .prefs(prefs)
             .outFilePath(extras != null ? extras.getString("out_file") : null)
+            .fds(fds)
             .build();
 
     if (info.xpcshell
@@ -471,7 +532,11 @@ public final class GeckoRuntime implements Parcelable {
 
     // Bug 1453062 -- the EventDispatcher should really live here (or in GeckoThread)
     EventDispatcher.getInstance()
-        .registerUiThreadListener(mEventListener, "Gecko:Exited", "GeckoView:Test:NewTab");
+        .registerUiThreadListener(
+            mEventListener,
+            "Gecko:Exited",
+            "GeckoView:Test:NewTab",
+            "GeckoView:ServiceWorkerOpenWindow");
 
     // Attach and commit settings.
     mSettings.attachTo(this);
@@ -1054,4 +1119,17 @@ public final class GeckoRuntime implements Parcelable {
           return new GeckoRuntime[size];
         }
       };
+
+  /**
+   * Whether the default `interactive-widget` is `resizes-visual`.
+   *
+   * @return True the default `interactive-widget` is `resizes-visual`, false otherwise.
+   */
+  @AnyThread
+  public boolean isInteractiveWidgetDefaultResizesVisual() {
+    if (!GeckoThread.isStateAtLeast(GeckoThread.State.JNI_READY)) {
+      return false;
+    }
+    return GeckoAppShell.isInteractiveWidgetDefaultResizesVisual();
+  }
 }
