@@ -23,7 +23,7 @@ let types = {
   // so it is used here for this test.
   js: "application/x-javascript",
   binary: "application/octet-stream",
-  gook: "application/x-gook",
+  nonsense: "application/x-nonsense",
   zip: "application/zip",
   json: "application/json",
   tar: "application/x-tar",
@@ -49,13 +49,13 @@ const WEBP_DATA = atob(
   "UklGRiIAAABXRUJQVlA4TBUAAAAvY8AYAAfQ/4j+B4CE8H+/ENH/VCIA"
 );
 
-const DEFAULT_INDEX_FILENAME =
-  AppConstants.platform == "win" ? "index.htm" : "index.html";
+const DEFAULT_FILENAME =
+  AppConstants.platform == "win" ? "Untitled.htm" : "Untitled.html";
 
 const PROMISE_FILENAME_TYPE = "application/x-moz-file-promise-dest-filename";
 
 let MockFilePicker = SpecialPowers.MockFilePicker;
-MockFilePicker.init(window);
+MockFilePicker.init(window.browsingContext);
 
 let expectedItems;
 let sendAsAttachment = false;
@@ -116,19 +116,6 @@ function handleRedirect(aRequest, aResponse) {
   aResponse.setHeader("Location", "/bell" + filename[0] + "?" + queryString);
 }
 
-function promiseDownloadFinished(list) {
-  return new Promise(resolve => {
-    list.addView({
-      onDownloadChanged(download) {
-        if (download.stopped) {
-          list.removeView(this);
-          resolve(download);
-        }
-      },
-    });
-  });
-}
-
 // nsIFile::CreateUnique crops long filenames if the path is too long, but
 // we don't know exactly how long depending on the full path length, so
 // for those save methods that use CreateUnique, instead just verify that
@@ -149,18 +136,18 @@ function checkShortenedFilename(actual, expected) {
   return false;
 }
 
-add_task(async function init() {
-  const { HttpServer } = ChromeUtils.import(
-    "resource://testing-common/httpd.js"
+add_setup(async function () {
+  const { HttpServer } = ChromeUtils.importESModule(
+    "resource://testing-common/httpd.sys.mjs"
   );
   httpServer = new HttpServer();
   httpServer.start(8000);
 
   // Need to load the page from localhost:8000 as the download attribute
   // only applies to links from the same domain.
-  let saveFilenamesPage = FileUtils.getFile(
-    "CurWorkD",
-    "/browser/uriloader/exthandler/tests/mochitest/save_filenames.html".split(
+  let saveFilenamesPage = await IOUtils.getFile(
+    Services.dirsvc.get("CurWorkD", Ci.nsIFile).path,
+    ..."browser/uriloader/exthandler/tests/mochitest/save_filenames.html".split(
       "/"
     )
   );
@@ -192,8 +179,8 @@ add_task(async function init() {
   expectedItems = await getItems("items");
 });
 
-function getItems(parentid) {
-  return SpecialPowers.spawn(
+async function getItems(parentid) {
+  let items = await SpecialPowers.spawn(
     gBrowser.selectedBrowser,
     [parentid, AppConstants.platform],
     (id, platform) => {
@@ -202,12 +189,13 @@ function getItems(parentid) {
       while (elem) {
         let filename =
           elem.dataset["filenamePlatform" + platform] || elem.dataset.filename;
-        let url = elem.getAttribute("src");
+        let url = elem.getAttribute("src") || elem.getAttribute("href");
         let draggable =
           elem.localName == "img" && elem.dataset.nodrag != "true";
         let unknown = elem.dataset.unknown;
         let noattach = elem.dataset.noattach;
         let savepagename = elem.dataset.savepagename;
+        let pickedfilename = elem.dataset.pickedfilename;
         elements.push({
           draggable,
           unknown,
@@ -215,12 +203,15 @@ function getItems(parentid) {
           url,
           noattach,
           savepagename,
+          pickedfilename,
         });
         elem = elem.nextElementSibling;
       }
       return elements;
     }
   );
+  Assert.greater(items.length, 0, "Some elements were found to test");
+  return items;
 }
 
 function getDirectoryEntries(dir) {
@@ -251,23 +242,13 @@ add_task(async function save_document() {
   tmpDir.append(baseFilename + "_document_files");
 
   MockFilePicker.displayDirectory = tmpDir;
-  MockFilePicker.showCallback = function(fp) {
+  MockFilePicker.showCallback = function () {
     MockFilePicker.setFiles([tmpFile]);
     MockFilePicker.filterIndex = 0; // kSaveAsType_Complete
   };
 
   let downloadsList = await Downloads.getList(Downloads.PUBLIC);
-  let savePromise = new Promise((resolve, reject) => {
-    downloadsList.addView({
-      onDownloadChanged(download) {
-        if (download.succeeded) {
-          downloadsList.removeView(this);
-          downloadsList.removeFinished();
-          resolve();
-        }
-      },
-    });
-  });
+  let savePromise = promiseDownloadFinished(downloadsList);
   saveBrowser(browser);
   await savePromise;
 
@@ -279,7 +260,7 @@ add_task(async function save_document() {
       // This is special-cased on Windows. The default filename will be used, since
       // the filename is invalid, but since the previous test file has the same issue,
       // this second file will be saved with a number suffix added to it.
-      filename = "index_002";
+      filename = "Untitled_002";
     }
 
     let file = tmpDir.clone();
@@ -301,8 +282,9 @@ add_task(async function save_document() {
       fileIdx = filesSaved.indexOf(filename);
     }
 
-    ok(
-      fileIdx >= 0,
+    Assert.greaterOrEqual(
+      fileIdx,
+      0,
       "file i" +
         idx +
         " " +
@@ -319,6 +301,7 @@ add_task(async function save_document() {
   is(filesSaved.length, 0, "all files accounted for");
   tmpDir.remove(true);
   tmpFile.remove(false);
+  downloadsList.removeFinished();
 });
 
 // This test simulates dragging the images in the document and ensuring that
@@ -392,7 +375,11 @@ if (AppConstants.platform != "macosx") {
 
 // This test checks that copying an image provides the right filename
 // for pasting to the local file system. This is only implemented on Windows.
-if (AppConstants.platform == "win") {
+const imageAsFileEnabled = SpecialPowers.getBoolPref(
+  "clipboard.imageAsFile.enabled",
+  false
+);
+if (AppConstants.platform == "win" && imageAsFileEnabled) {
   add_task(async function copy_image() {
     for (let idx = 0; idx < expectedItems.length; idx++) {
       if (!expectedItems[idx].draggable) {
@@ -400,30 +387,22 @@ if (AppConstants.platform == "win") {
         continue;
       }
 
-      let data = await SpecialPowers.spawn(
-        gBrowser.selectedBrowser,
-        [idx, PROMISE_FILENAME_TYPE],
-        (imagenum, type) => {
-          // No need to wait for the data to be really on the clipboard, we only
-          // need the promise data added when the command is performed.
-          SpecialPowers.setCommandNode(
-            content,
-            content.document.getElementById("i" + imagenum)
-          );
-          SpecialPowers.doCommand(content, "cmd_copyImageContents");
-
-          return SpecialPowers.getClipboardData(type);
-        }
-      );
-
-      is(
-        data,
+      await SimpleTest.promiseClipboardChange(
         expectedItems[idx].filename,
-        "i" +
-          idx +
-          " " +
-          expectedItems[idx].filename +
-          " was saved with the correct name when copying"
+        () => {
+          return SpecialPowers.spawn(
+            gBrowser.selectedBrowser,
+            [idx],
+            imagenum => {
+              SpecialPowers.setCommandNode(
+                content,
+                content.document.getElementById("i" + imagenum)
+              );
+              SpecialPowers.doCommand(content, "cmd_copyImageContents");
+            }
+          );
+        },
+        PROMISE_FILENAME_TYPE
       );
     }
   });
@@ -462,23 +441,38 @@ add_task(async function saveas_files() {
           let list = await Downloads.getList(Downloads.PUBLIC);
           let downloadFinishedPromise = promiseDownloadFinished(list);
 
-          await BrowserTestUtils.openNewForegroundTab(
+          await BrowserTestUtils.openNewForegroundTab({
             gBrowser,
-            expectedItems[idx].url
-          );
+            opening: expectedItems[idx].url,
+            waitForLoad: false,
+            waitForStateStop: true,
+          });
 
           let download = await downloadFinishedPromise;
 
           let filename = PathUtils.filename(download.target.path);
-          is(
-            filename,
-            expectedItems[idx].filename,
-            "open link" +
-              idx +
-              " " +
-              expectedItems[idx].filename +
-              " was downloaded with the correct name when opened as a url"
-          );
+
+          let expectedFilename = expectedItems[idx].filename;
+          if (expectedFilename.length > 240) {
+            ok(
+              checkShortenedFilename(filename, expectedFilename),
+              "open link" +
+                idx +
+                " " +
+                expectedFilename +
+                " was downloaded with the correct name when opened as a url (with long name)"
+            );
+          } else {
+            is(
+              filename,
+              expectedFilename,
+              "open link" +
+                idx +
+                " " +
+                expectedFilename +
+                " was downloaded with the correct name when opened as a url"
+            );
+          }
 
           try {
             await IOUtils.remove(download.target.path);
@@ -488,14 +482,16 @@ add_task(async function saveas_files() {
           continue;
         }
 
-        await BrowserTestUtils.openNewForegroundTab(
+        await BrowserTestUtils.openNewForegroundTab({
           gBrowser,
-          expectedItems[idx].url
-        );
+          opening: expectedItems[idx].url,
+          waitForLoad: false,
+          waitForStateStop: true,
+        });
       }
 
       let filename = await new Promise(resolve => {
-        MockFilePicker.showCallback = function(fp) {
+        MockFilePicker.showCallback = function (fp) {
           setTimeout(() => {
             resolve(fp.defaultString);
           }, 0);
@@ -512,9 +508,9 @@ add_task(async function saveas_files() {
 
       // Trying to open an unknown or binary type will just open a blank
       // page, so trying to save will just save the blank page with the
-      // filename index.html.
+      // filename Untitled.html.
       let expectedFilename = expectedItems[idx].unknown
-        ? DEFAULT_INDEX_FILENAME
+        ? DEFAULT_FILENAME
         : expectedItems[idx].savepagename || expectedItems[idx].filename;
 
       // When saving via contentAreaUtils.js, the content disposition name
@@ -541,6 +537,59 @@ add_task(async function saveas_files() {
   }
 });
 
+// This test checks that the filename is saved correctly when it
+// has been modified within the file picker.
+add_task(async function saveas_files_modified_in_filepicker() {
+  let items = await getItems("modifieditems");
+  for (let idx = 0; idx < items.length; idx++) {
+    await BrowserTestUtils.openNewForegroundTab({
+      gBrowser,
+      opening: items[idx].url,
+      waitForLoad: false,
+      waitForStateStop: true,
+    });
+
+    let savedFile = SpecialPowers.Services.dirsvc.get("TmpD", Ci.nsIFile);
+
+    let downloadsList = await Downloads.getList(Downloads.PUBLIC);
+    let savePromise = promiseDownloadFinished(downloadsList);
+
+    await new Promise(resolve => {
+      MockFilePicker.displayDirectory = savedFile;
+
+      MockFilePicker.showCallback = function () {
+        MockFilePicker.filterIndex = 0; // kSaveAsType_Complete
+        savedFile.append(items[idx].pickedfilename);
+        MockFilePicker.setFiles([savedFile]);
+        setTimeout(() => {
+          resolve(items[idx].pickedfilename);
+        }, 0);
+
+        return Ci.nsIFilePicker.returnOK;
+      };
+
+      document.getElementById("Browser:SavePage").doCommand();
+    });
+
+    await savePromise;
+
+    savedFile.leafName = items[idx].filename;
+    ok(
+      savedFile.exists(),
+      "i" +
+        idx +
+        " '" +
+        savedFile.leafName +
+        "' was saved when modified with the correct name "
+    );
+    if (savedFile.exists()) {
+      savedFile.remove(false);
+    }
+
+    await BrowserTestUtils.removeTab(gBrowser.selectedTab);
+  }
+});
+
 // This test checks that links that result in files with
 // content-disposition: attachment are saved with the right filenames.
 add_task(async function save_links() {
@@ -641,7 +690,7 @@ add_task(async function saveas_image_links() {
       await popupShown;
 
       let promptPromise = new Promise(resolve => {
-        MockFilePicker.showCallback = function(fp) {
+        MockFilePicker.showCallback = function (fp) {
           setTimeout(() => {
             resolve(fp.defaultString);
           }, 0);
@@ -722,11 +771,11 @@ add_task(async function save_download_links() {
           " was saved with the correct name when link has download attribute"
       );
     } else {
-      if (idx == 66 && filename == "index(1)") {
+      if (idx == 66 && filename == "Untitled(1)") {
         // Sometimes, the previous test's file still exists or wasn't created in time
         // and a non-duplicated name is created. Allow this rather than figuring out
         // how to avoid it since it doesn't affect what is being tested here.
-        filename = "index";
+        filename = "Untitled";
       }
 
       is(
@@ -746,6 +795,33 @@ add_task(async function save_download_links() {
   }
 });
 
+// This test verifies that invalid extensions are not removed when they
+// have been entered in the file picker.
+add_task(async function save_page_with_invalid_after_filepicker() {
+  await BrowserTestUtils.openNewForegroundTab(
+    gBrowser,
+    "http://localhost:8000/save_filename.sjs?type=html&filename=invfile.lnk"
+  );
+
+  let filename = await new Promise(resolve => {
+    MockFilePicker.showCallback = function (fp) {
+      let expectedFilename =
+        AppConstants.platform == "win" ? "invfile.lnk.htm" : "invfile.lnk.html";
+      is(fp.defaultString, expectedFilename, "supplied filename is correct");
+      setTimeout(() => {
+        resolve("otherfile.local");
+      }, 0);
+      return Ci.nsIFilePicker.returnCancel;
+    };
+
+    document.getElementById("Browser:SavePage").doCommand();
+  });
+
+  is(filename, "otherfile.local", "lnk extension has been preserved");
+
+  await BrowserTestUtils.removeTab(gBrowser.selectedTab);
+});
+
 add_task(async function save_page_with_invalid_extension() {
   await BrowserTestUtils.openNewForegroundTab(
     gBrowser,
@@ -753,7 +829,7 @@ add_task(async function save_page_with_invalid_extension() {
   );
 
   let filename = await new Promise(resolve => {
-    MockFilePicker.showCallback = function(fp) {
+    MockFilePicker.showCallback = function (fp) {
       setTimeout(() => {
         resolve(fp.defaultString);
       }, 0);
